<?php

/**
 * webEdition CMS
 *
 * $Rev: 64aba95e1ae5 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-01-24 13:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_info extends we_editor_base{

	public function show(){

		$GLOBALS['we_transaction'] = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', we_base_request::_(we_base_request::TRANSACTION, 'we_transaction'), 2);
		if($this->we_doc->ContentType !== we_base_ContentTypes::FOLDER && $this->we_doc->ContentType !== we_base_ContentTypes::COLLECTION){
			$fs = $this->we_doc->getFilesize();
		}

		$parts = [
			['headline' => 'ID',
				'html' => ($this->we_doc->ID ?: '-'),
				'icon' => we_html_element::span(['class' => 'docIcon', 'data-contenttype' => $this->we_doc->ContentType, 'data-extension' => (isset($this->we_doc->Extension) ? $this->we_doc->Extension : '')]),
				'noline' => true,
				'rightHeadline' => true
			],
			['headline' => g_l('weEditorInfo', '[content_type]'),
				'html' => ($this->we_doc->ContentType ? g_l('weEditorInfo', '[' . $this->we_doc->ContentType . ']') : ''),
				'space' => we_html_multiIconBox::SPACE_ICON,
				'noline' => true,
				'rightHeadline' => true
			],
			['headline' => g_l('weEditorInfo', '[file_size]'),
				'html' => (isset($fs) ? round(($fs / 1024), 2) . "&nbsp;KB&nbsp;(" . $fs . "&nbsp;Byte)" : ''),
				'space' => we_html_multiIconBox::SPACE_ICON,
				'rightHeadline' => true,
				'hide' => !isset($fs),
				'noline' => true,
			],
			['headline' => g_l('weEditorInfo', '[creation_date]'),
				'html' => date(g_l('weEditorInfo', '[date_format]'), $this->we_doc->CreationDate),
				'icon' => we_html_multiIconBox::INFO_CALENDAR,
				'noline' => true,
				'preline' => true
			],
			['headline' => g_l('modules_users', '[created_by]'),
				'html' => f('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->we_doc->CreatorID)) ?: '-',
				'space' => we_html_multiIconBox::SPACE_ICON,
				'noline' => true,
				'rightHeadline' => true,
				'hide' => empty($this->we_doc->CreatorID)
			],
			['headline' => g_l('weEditorInfo', '[changed_date]'),
				'html' => date(g_l('weEditorInfo', '[date_format]'), $this->we_doc->ModDate),
				'space' => we_html_multiIconBox::SPACE_ICON,
				'noline' => true,
				'rightHeadline' => true
			],
			['headline' => g_l('modules_users', '[changed_by]'),
				'html' => f('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->we_doc->ModifierID)) ?: '-',
				'space' => we_html_multiIconBox::SPACE_ICON,
				'noline' => true,
				'rightHeadline' => true,
				'hide' => empty($this->we_doc->ModifierID)
			],
			['headline' => g_l('weEditorInfo', '[lastLive]'),
				'html' => ($this->we_doc->Published ? date(g_l('weEditorInfo', '[date_format]'), $this->we_doc->Published) : "-") .
				(!in_array($this->we_doc->Table, [TEMPLATES_TABLE, VFILE_TABLE]) && $this->we_doc->ContentType !== we_base_ContentTypes::FOLDER && $this->we_doc->Published && $this->we_doc->ModDate > $this->we_doc->Published ?
				'<div style="margin-bottom:10px;">' . we_html_button::create_button('revert_published', "javascript:top.we_cmd('revert_published_question');") . '</div>' :
				''),
				'space' => we_html_multiIconBox::SPACE_ICON,
				'noline' => true,
				'hide' => !in_array($this->we_doc->ContentType, [we_base_ContentTypes::HTML, we_base_ContentTypes::WEDOCUMENT]),
				'rightHeadline' => true
			],
		];

		$this->jsCmd->addCmd('setIconOfDocClass', 'docIcon');

		if($this->we_doc->ContentType !== we_base_ContentTypes::FOLDER){
			switch($this->we_doc->Table){
				case TEMPLATES_TABLE:
				case VFILE_TABLE:
					break;
				default:
					$rp = realpath($this->we_doc->getRealPath());
					$http = $this->we_doc->getHttpPath();

					switch($this->we_doc->ContentType){
						default:
							$showlink = false;
							break;
						case we_base_ContentTypes::WEDOCUMENT:
							$showlink = true;
							if(we_modules_info::isActive(we_modules_info::WORKFLOW)){
								$anzeige = (we_workflow_utility::inWorkflow($this->we_doc->ID, $this->we_doc->Table) ?
									we_workflow_utility::getDocumentStatusInfo($this->we_doc->ID, $this->we_doc->Table) :
									we_workflow_utility::getLogButton($this->we_doc->ID, $this->we_doc->Table));
							}
							break;

						case we_base_ContentTypes::HTML:
						case we_base_ContentTypes::IMAGE:
						case we_base_ContentTypes::FLASH:
						case we_base_ContentTypes::VIDEO:
						case we_base_ContentTypes::AUDIO:
						case we_base_ContentTypes::PDF:
							$showlink = true;
					}

					$published = !(($this->we_doc->ContentType === we_base_ContentTypes::HTML || $this->we_doc->ContentType === we_base_ContentTypes::WEDOCUMENT) && $this->we_doc->Published < 1 );

					$parts[] = ['headline' => g_l('weEditorInfo', '[local_path]'),
						'html' => ($this->we_doc->ID == 0 || !$published ? '-' : '<span title="' . oldHtmlspecialchars($rp) . '">' . oldHtmlspecialchars(we_base_util::shortenPath($rp, 74)) . '</span>'),
						'icon' => we_html_multiIconBox::PROP_PATH,
						'noline' => true,
						'preline' => true
					];
					$parts[] = ['headline' => g_l('weEditorInfo', '[http_path]'),
						'html' => ($this->we_doc->ID == 0 || !$published ? '-' : ($showlink ? '<a href="' . $http . '" target="_blank" title="' . oldHtmlspecialchars($http) . '">' : '') . $http . ($showlink ? '</a>' : '')),
						'space' => we_html_multiIconBox::SPACE_ICON,
						'noline' => true,
						'rightHeadline' => true
					];
			}

			if(isset($anzeige)){
				$parts[] = ['headline' => g_l('modules_workflow', '[workflow]'),
					'html' => $anzeige,
					'rightHeadline' => true,
					'icon' => we_html_multiIconBox::INFO_WORKFLOW,
					'preline' => true,
					'noline' => true,
				];
			}

			if($this->we_doc->isBinary()){
				$formReference = $this->we_doc->formReferences($this->jsCmd);
				$parts[] = [
					'headline' => g_l('weClass', '[isUsed]') . ' (' . $formReference['num'] . ')',
					'html' => $formReference['form'],
					'rightHeadline' => true,
					'icon' => we_html_multiIconBox::INFO_REFERENCES,
					'preline' => true,
				];
			}

			switch($this->we_doc->ContentType){
				case we_base_ContentTypes::TEMPLATE:
					list($cnt, $select) = $this->we_doc->formTemplateDocuments();
					$parts[] = [
						'icon' => we_html_multiIconBox::INFO_REFERENCES,
						'headline' => g_l('weClass', '[documents]') . ($cnt ? ' (' . $cnt . ')' : ''),
						'html' => $select,
						'noline' => true,
						'preline' => true,
					];
					list($cnt, $select) = $this->we_doc->formTemplatesUsed();
					$parts[] = [
						'headline' => g_l('weClass', '[usedTemplates]') . ($cnt ? ' (' . $cnt . ')' : ''),
						'html' => $select,
						'space' => we_html_multiIconBox::SPACE_ICON,
						'noline' => true,
						'rightHeadline' => true
					];
					list($cnt, $select) = $this->we_doc->formTemplateUsedByTemplate();
					$parts[] = [
						'headline' => g_l('weClass', '[usedByTemplates]') . ($cnt ? ' (' . $cnt . ')' : ''),
						'html' => $select,
						'space' => we_html_multiIconBox::SPACE_ICON,
						'noline' => true,
						'rightHeadline' => true
					];
					break;
				case we_base_ContentTypes::IMAGE:
					$metaData = $this->we_doc->getMetaData();
					$metaDataTable = '<table class="default">';
					if(empty($metaData['exif'])){
						$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont" colspan="2">' . g_l('metadata', (is_callable("exif_read_data") ? '[no_exif_data]' : '[no_exif_installed]')) . '</td></tr>';
					} else {
						foreach($metaData['exif'] as $key => $val){
							$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont">' . oldHtmlspecialchars($key) . ':</td><td style="padding:0 5px 5px 0;" class="defaultfont">' . oldHtmlspecialchars($val) . '</td></tr>';
						}
					}
					$metaDataTable .= '</table>';
					$parts[] = [
						'headline' => g_l('metadata', '[info_exif_data]'),
						'html' => $metaDataTable,
						'rightHeadline' => true,
						'icon' => we_html_multiIconBox::PROP_META
					];

					$metaDataTable = '<table class="default">';
					if(empty($metaData['iptc'])){
						$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont" colspan="2">' . g_l('metadata', '[no_iptc_data]') . '</td></tr>';
					} else {
						foreach($metaData['iptc'] as $key => $val){
							$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont">' . oldHtmlspecialchars($key) . ':</td><td style="padding:0 5px 5px 0;" class="defaultfont">' . oldHtmlspecialchars($val) . '</td></tr>';
						}
					}
					$metaDataTable .= '</table>';
					$parts[] = [
						'headline' => g_l('metadata', '[info_iptc_data]'),
						'html' => $metaDataTable,
						'rightHeadline' => true,
						'icon' => we_html_multiIconBox::PROP_META
					];
					break;
				case we_base_ContentTypes::PDF:
					$metaData = $this->we_doc->getMetaData();
					$metaDataTable = '<table class="default">';
					if(!empty($metaData['pdf'])){
						foreach($metaData['pdf'] as $key => $val){
							$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont">' . oldHtmlspecialchars($key) . ':</td><td style="padding:0 5px 5px 0;" class="defaultfont">' . oldHtmlspecialchars($val) . '</td></tr>';
						}
					}
					$metaDataTable .= '</table>';
					$parts[] = [
						'headline' => g_l('metadata', '[info_pdf_data]'),
						'html' => $metaDataTable,
						'rightHeadline' => true,
						'icon' => we_html_multiIconBox::PROP_META
					];
					break;

				case we_base_ContentTypes::APPLICATION:
				default:
					if($this->we_doc->isBinary()){
						//$metaDataTable = g_l('metadata', '[no_metadata_supported]');
					}
			}
		}
		return $this->getPage(we_html_multiIconBox::getHTML('', $parts, true));
	}

}
