<?php

/**
 * webEdition CMS
 *
 * $Rev: c3b66c63fe6b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-28 11:50 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_navigation_dynList{

	public static function getDocuments($doctypeid, $dirID, array $categories, $catlogic, $order, $count, $field){
		$db = new DB_WE();
		$cats = [];
		foreach(array_filter($categories) as $cat){
			$cats[] = 'FIND_IN_SET(' . $cat . ',f.Category)'; //bug #6729
		}
		$sort = empty($order) ? [] : $order[0];

		return $db->getAllq('SELECT f.ID AS id,f.Text AS text,COALESCE(c.Dat,c.BDID) AS field FROM ' . FILE_TABLE . ' f JOIN ' . CONTENT_TABLE . ' c ON (f.ID=c.DID AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '" AND c.nHash=x\'' . md5($field ?: 'Title') . '\') ' .
						($sort ? ' JOIN ' . CONTENT_TABLE . ' cs ON (cs.DID=f.ID AND cs.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '")' : ''
						) . 'WHERE (f.IsFolder=0 AND f.Published>0) ' .
						($dirID ? ' AND f.ParentID=' . $dirID : '') .
						($doctypeid ? ' AND f.DocType=' . $db->escape($doctypeid) : '') .
						($cats ? (' AND (' . implode(' ' . $catlogic . ' ', $cats) . ')') : '') .
						($sort ? (' AND cs.nHash=x\'' . md5($sort['field']) . '\' ORDER BY COALESCE(cs.Dat,cs.BDID) ' . $sort['order']) : '') .
						'  LIMIT ' . $count);
	}

	public static function getObjects($classid, $dirid, array $categories, $catlogic, array $sorts, $count, $field){
		$select = 'of.ID,of.Text' . ($field ? ',obx.' . $field : '');
		$order = [];
		foreach($sorts as $sort){
			$order[] = $sort['field'] . ' ' . $sort['order'];
		}
		$fieldset = self::getObjData($select, $classid, id_to_path($dirid, OBJECT_FILES_TABLE), $categories, $catlogic, $order, $count);
		$ids = [];

		foreach($fieldset as $data){
			$ids[] = [
				'id' => $data['ID'],
				'text' => $data['Text'],
				'field' => !empty($data[$field]) ? we_navigation_navigation::encodeSpecChars($data[$field]) : ''
			];
		}

		return $ids;
	}

	private static function getObjData($select, $classid, $dirpath, array $categories, $catlogic, array $order, $count){
		$db = new DB_WE();
		$cats = [];
		foreach(array_filter($categories) as $cat){
			$cats[] = 'FIND_IN_SET(' . $cat . ',of.Category)'; //bug #6729
		}

		$where = [];

		if($cats){
			$where[] = '(' . implode($catlogic, $cats) . ')';
		}

		if($dirpath != '/'){
			$where[] = 'of.Path LIKE "' . $db->escape($dirpath) . '%"';
		}
		$where[] = 'of.Published>0'; // Bug #4797
		return $db->getAllq('SELECT ' . $select . ' FROM ' . OBJECT_X_TABLE . intval($classid) . ' obx JOIN ' . OBJECT_FILES_TABLE . ' `of` ON obx.OF_ID=of.ID WHERE of.ID!=0 ' .
						($where ? ('AND ' . implode(' AND ', $where)) : '') .
						($order ? (' ORDER BY ' . implode(',', $order)) : '') . ' LIMIT ' . $count);
	}

	public static function getCatgories($dirid, $count){
		$ids = [];
		$db = $GLOBALS['DB_WE'];
		$db->query('SELECT ID as id,Text as text,Title as field FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . intval($dirid) . ' LIMIT ' . $count);

		while($db->next_record(MYSQLI_ASSOC)){
			$ids[] = [
				'id' => $db->f('id'),
				'text' => $db->f('text'),
				'field' => we_navigation_navigation::encodeSpecChars($db->f('field'))
			];
		}

		return $ids;
	}

	public static function getWorkspacesForObject($id){
		$hash = getHash('SELECT of.Workspaces,o.Workspaces FROM ' . OBJECT_FILES_TABLE . ' `of` JOIN ' . OBJECT_TABLE . ' o ON of.TableID=o.ID WHERE of.ID=' . intval($id), null, MYSQLI_NUM);
		if(!$hash){
			return [];
		}
		list($obWsp, $clWsp) = $hash;
		$ret = id_to_path($obWsp, FILE_TABLE, null, true);
		$all = we_objectFile::getPossibleWorkspaces($clWsp);

		return array_intersect_key($ret, $all);
	}

	public static function getWorkspacesForClass($id){
		$values = f('SELECT Workspaces FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($id));
		return id_to_path($values, FILE_TABLE, null, true);
	}

	public static function getFirstDynDocument($id){
		return f('SELECT ID FROM ' . FILE_TABLE . ' WHERE ParentID=' . intval($id) . ' AND IsFolder=0 AND IsDynamic=1 AND Published!=0 LIMIT 1');
	}

	public static function getWorkspaceFlag($id){
		$clsid = f('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($id));
		$cls = new we_object();
		$cls->initByID($clsid, OBJECT_TABLE);

		return $cls->WorkspaceFlag;
	}

}
