<?php
/**
 * webEdition CMS
 *
 * $Rev: f391cb5474a7 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-09 16:54 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * @name we_selectorQuery
 */
class we_selector_query{
	/*	 * ***********************************************************************
	 * VARIABLES
	 * *********************************************************************** */
	private $db;
	var $result = [];
	var $fields;
	var $condition = [];

	/*	 * ***********************************************************************
	 * CONSTRUCTOR
	 * *********************************************************************** */

	/**
	 * Constructor of class
	 *
	 */
	public function __construct(){
		$this->db = new DB_WE();
		$this->fields = ['ID', 'Path'];
	}

	/**
	 * query
	 * Die Funktion 'query' f�hrt die Abfrage f�r den �bergebenen Selektor-Typ durch.
	 * Mit dem Parameter 'limit' kann man die Anzahl der Suchergebnisse begrenzen.
	 *
	 * @param search
	 * @param table
	 * @param types
	 * @param limit
	 *
	 * @return void
	 */
	function queryTable($search, $table, array $types, $limit = null){
		$userExtraSQL = $this->getUserExtraQuery($table);
		$isFolder = 1;

		switch($table){
			case USER_TABLE:
				$this->addQueryField('IsFolder');
				$typeField = 'Type';
				break;
			case CUSTOMER_TABLE :
				$typeField = 'ContentType';
				$userExtraSQL = '';
				break;
			case CATEGORY_TABLE:
				$isFolder = 0;
				break;
			case VFILE_TABLE:
			case NEWSLETTER_TABLE :
				break;
			default:
				$typeField = "ContentType";
		}

		$addCT = 0;
		$types = array_unique(array_filter($types));

		$q = [];
		foreach($types as $type){
			$type = str_replace(" ", "", $type);
			if($type == we_base_ContentTypes::FOLDER){
				$q[] = 'IsFolder=1';
			} elseif(!empty($typeField)){
				$q[] = $typeField . '="' . $this->db->escape($type) . '"';
				$isFolder = 0;
				$addCT = 1;
			}
		}

		if($addCT){
			$this->addQueryField($typeField);
		}
		$where = 'Path="' . $this->db->escape($search) . '"' .
			($q ? ' AND (' . implode(' OR ', $q) . ')' : '') .
			($userExtraSQL ?: '');

		if($this->condition){
			foreach($this->condition as $val){
				$where .= ' ' . $val['queryOperator'] . ' ' . $val['field'] . $val['conditionOperator'] . "'" . $val['value'] . "'";
			}
		}

		$order = 'ORDER BY ' . ($isFolder ? 'isFolder ASC,Path' : 'Path' ) . ' ASC ';
		$fields = implode(', ', $this->fields);
		$this->db->query('SELECT ' . $fields . ' FROM ' . $this->db->escape($table) . ' WHERE ' . $where . ' ' . $order . ($limit ? ' LIMIT ' . $limit : ''));
	}

	/**
	 * search
	 * Die Funktion 'search' f�hrt die Suche nach den Anfangszeichen f�r den �bergebenen Selektor-Typ durch.
	 * Mit dem Parameter 'limit' kann man die Anzahl der Suchergebnisse begrenzen.
	 *
	 * @param search
	 * @param table
	 * @param types
	 * @param limit
	 *
	 * @return void
	 */
	function search($search, $table, array $types, $limit = 0, $rootDir = ''){
		$search = strtr($search, ['[' => "\\\[", ']' => "\\\]"]);
		$userExtraSQL = $this->getUserExtraQuery($table);
		switch($table){
			case USER_TABLE:
				$this->addQueryField('IsFolder');
				$typeField = 'Type';
				$folderType = we_users_user::TYPE_USER_GROUP;
				break;
			case CUSTOMER_TABLE :
				$typeField = 'ContentType';
				$folderType = we_base_ContentTypes::FOLDER;
				$userExtraSQL = '';
				break;
			case VFILE_TABLE:
			case CATEGORY_TABLE:
			case NEWSLETTER_TABLE:
			case NAVIGATION_TABLE:
				break;
			default:
				$typeField = 'ContentType';
				$folderType = we_base_ContentTypes::FOLDER;
		}

		$rootOnly = $rootDir && ($search === '/' || strpos($rootDir, $search) === 0);
		$where = ($rootOnly ?
			'Path LIKE "' . $rootDir . '"' :
			'Path REGEXP "^' . preg_quote(strtr($search, ['"' => ''])) . '[^/]*$"' . (
			($rootDir != '/' ?
			' AND (Path LIKE "' . $this->db->escape($rootDir) . '" OR Path LIKE "' . $this->db->escape($rootDir) . '%")' :
			'')));

		$isFolder = 0;
		$addCT = 0;

		$q = [];
		if(!empty($typeField)){
			foreach(array_unique($types) as $type){
				$type = str_replace(' ', '', $type);
				if($type === we_base_ContentTypes::FOLDER){
					$q[] = '"' . $folderType . '"';
				} else {
					$q[] = '"' . $this->db->escape($type) . '"';
					$isFolder = 0;
					$addCT = 1;
				}
			}
		}
		$where .= ($q ? ' AND (' . $typeField . ' IN(' . implode(',', $q) . '))' : '');
		if(!$q){
			$isFolder = 1;
		}
		switch($table){
			case CATEGORY_TABLE:
				$isFolder = 0;
				break;
		}
		if($addCT){
			$this->addQueryField($typeField);
		}
		$where .= $userExtraSQL;

		if(!empty($this->condition)){
			foreach($this->condition as $val){
				$where .= ' ' . $val['queryOperator'] . ' ' . $val['field'] . $val['conditionOperator'] . '"' . $this->db->escape($val['value']) . '"';
			}
		}
		if($table === NAVIGATION_TABLE){
			$where .= we_navigation_navigation::getWSQuery();
		}
		//FIXME: what about other workspacequeries?!
		$this->db->query('SELECT ' . implode(', ', $this->fields) . ' FROM ' . $this->db->escape($table) . ' WHERE ' . $where . ' ORDER BY ' . ($isFolder ? 'isFolder DESC, Path' : 'Path') . ' ASC ' . ($limit ? ' LIMIT ' . $limit : ''));
	}

	/**
	 * Returns all entries of a folder, depending on the contenttype.
	 *
	 * @param integer $id
	 * @param string $table
	 * @param array $types
	 * @param integer $limit
	 */
	function queryFolderContents($id, $table, $types = null, $limit = null){
		$userExtraSQL = $this->getUserExtraQuery($table);
		if(is_array($types) && $table !== CATEGORY_TABLE){
			$this->addQueryField('ContentType');
		}

		$this->addQueryField('Text');
		$this->addQueryField('ParentID');

		// deal with contenttypes
		if($table === CATEGORY_TABLE){
			$types = [];
		}

		$this->db->query('SELECT ' . implode(',', $this->fields) . ' FROM ' . $this->db->escape($table) . ' WHERE ParentID=' . intval($id) . ' AND (IsFolder=1 ' . ($ctntQuery ? ' OR ContentType IN ("' . implode('","', $types) . '")' : '') . ' ) ' .
			$userExtraSQL . ' ORDER BY IsFolder DESC, Path ');
	}

	/**
	 * returns single item by id
	 *
	 * @param integer $id
	 * @param string $table
	 */
	function getItemById($id, $table, $fields = '', $useExtraSQL = true){
		$userExtraSQL = ($table !== BANNER_TABLE ?
			($useExtraSQL ? $this->getUserExtraQuery($table) : '') : '');

		$this->addQueryField('Text');
		$this->addQueryField('ParentID');
		if(is_array($fields)){
			foreach($fields as $val){
				$this->addQueryField($val);
			}
		}
		$this->db->query('SELECT ' . implode(',', $this->fields) . ' FROM ' . $this->db->escape($table) . ' WHERE ID=' . intval($id) . ' ' . $userExtraSQL);
		return $this->getResult();
	}

	/**
	 * returns single item by id
	 *
	 * @param integer $id
	 * @param string $table
	 */
	function getItemByPath($path, $table, $fields = ''){
		$userExtraSQL = $this->getUserExtraQuery($table);

		$this->addQueryField('Text');
		$this->addQueryField('ParentID');
		if(is_array($fields)){
			foreach($fields as $val){
				$this->addQueryField($val);
			}
		}
		$this->db->query('SELECT ' . implode(',', $this->fields) . ' FROM ' . $this->db->escape($table) . ' WHERE Path="' . $this->db->escape($path) . '" ' . $userExtraSQL);
		return $this->getResult();
	}

	/**
	 * getResult
	 * Liefert das komplette Erg�bnis der Abfrage als Hash mit den Feldnamen als Spalten.
	 * @return Array
	 */
	function getResult(){
		return $this->db->getAll();
	}

	/**
	 * addQueryField
	 * F�gt den �bergebenen String zur Liste der gesuchten Felder hinzu.
	 * @param field
	 * @return void
	 */
	function addQueryField($field){
		$this->fields[] = $field;
	}

	/**
	 * addCondition
	 * F�gt die �bergeben Abfragebedingung hinzu.
	 * @param array $condition
	 */
	function addCondition($qop, $op, $field, $value){
		$this->condition[] = [
			'queryOperator' => $qop,
			'conditionOperator' => $op,
			'field' => $field,
			'value' => $value
		];
	}

	/**
	 * getUserExtraQuery
	 * Erzeugt ein Bedingungen zur Filterung der Arbeitsbereiche
	 * @param string $table
	 * @return string
	 */
	function getUserExtraQuery($table){
		switch($table){
			case NAVIGATION_TABLE:
			case BANNER_TABLE:
			case CATEGORY_TABLE:
				return '';
		}
		$userExtraSQL = ' AND((1 ' . we_users_util::makeOwnersSql(false) . ') ';

		if(get_ws($table)){
			$userExtraSQL .= we_selector_file::getWsQuery($table);
		} else if($table === OBJECT_FILES_TABLE && (!we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR))){
			$wsQuery = [];
			$ac = we_users_util::getAllowedClasses($this->db);
			foreach($ac as $cid){
				$path = id_to_path($cid, OBJECT_TABLE);
				$wsQuery[] = ' Path LIKE "' . $this->db->escape($path) . '/%" OR ID=' . $cid;
			}
			if($wsQuery){
				$userExtraSQL .= ' AND (' . implode(' OR ', $wsQuery) . ')';
			}
		} else {
			switch($table){
				case (we_modules_info::isActive(we_modules_info::NEWSLETTER) ? NEWSLETTER_TABLE : 'NEWSLETTER_TABLE'):
				case USER_TABLE:
					break;
				default:
					$userExtraSQL .= ' OR RestrictOwners=0 ';
			}
		}
		return $userExtraSQL . ')';
	}

}
