<?php

/**
 * webEdition CMS
 *
 * $Rev: 22a81a512887 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-09-17 16:44 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_ifEmpty extends we_weTag_ifTag{

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(!empty($GLOBALS['we']['editmode'])){
			return false;
		}

		$docAttr = self::getAttribute('doc', $attribs, '', we_base_request::STRING);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING) ?: self::getPostName(self::getAttribute('match', $attribs, '', we_base_request::STRING));
		$doc = self::getDocForTag($docAttr, false);
		$regs = [];

		if(!$name && ($foo = self::attributeMissing($attribs, 'name', __CLASS__))){
			echo $foo;
			return false;
		}

		if(preg_match('|^\\\\?\$([^\[]+)(\[.*\])?|', $name, $regs)){
			$name = (isset($regs[2]) ?
				self::getArrayValue($GLOBALS, $regs[1], $regs[2]) :
				(isset($GLOBALS[$regs[1]]) ? $GLOBALS[$regs[1]] : ''));
		}

		switch(self::getAttribute('type', $attribs, '', we_base_request::STRING)){
			case 'checkbox':
			case 'object':
				$val = $doc->getElement($name);
				return empty($val);
			case 'binary' :
			case 'img':
			case 'flashmovie' :
				$val = $doc->getElement($name, we_contents_base::ETYPE_NUMBER);
				return !($val && id_to_path([$val], FILE_TABLE, $GLOBALS['DB_WE'], false, false, true));
			case 'href':
				if(isset($doc->OF_ID)){
					$hreftmp = $doc->getElement($name);
					if(!$hreftmp || !is_array($hreftmp)){
						return true;
					}
					$hreftmp = trim(we_contents_document::getHrefByArray($hreftmp));

					return ($hreftmp && $hreftmp{0} === '/' ?
						(!file_exists(WEBEDITION_PATH . '../' . $hreftmp)) :
						($hreftmp ? false : true)
						);
				}
				if(intval($doc->getElement($name . we_base_link::MAGIC_INT_LINK, we_contents_base::ETYPE_NUMBER))){ // for type = href int
					$intID = $doc->getElement($name . we_base_link::MAGIC_INT_LINK_ID, we_contents_base::ETYPE_NUMBER);
					return !($intID && id_to_path([$intID], FILE_TABLE, $GLOBALS['DB_WE'], false, false, true));
				}
				$hreftmp = $doc->getElement($name);
				if($hreftmp && $hreftmp{0} === '/'){
					return (!file_exists(WEBEDITION_PATH . '../' . $hreftmp));
				}

			default:
				$dat = $doc->getElement($name, we_contents_base::ETYPE_TEXT);
				//   #3938 added this - some php version crashed, when unserialize started with a ?,?,?
				if($dat && (substr($dat, 0, 2) === 'a:' || $dat{0} === '{')){ //  only unserialize, when $match could be an array
					// Added @-operator in front of the unserialze function because there
					// were some PHP notices that had no effect on the output of the function
					// remark holeg: when it is a serialized array, the function looks if it is not empty
					if(is_array($arr = we_unserialize($dat))){
						return empty($arr);
					}
				}

			//   end of #3938
		}

		$val1 = $doc->getElement($name);
		$val2 = $doc->getElement($name, we_contents_base::ETYPE_NUMBER);
		return empty($val1) && empty($val2);
	}

}

function we_tag_ifEmpty(array $attribs){
	return we_tag_ifEmpty::tag($attribs);
}
